package fr.mael.jiwigo.ui.search.tree;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import fr.mael.jiwigo.transverse.ImagesManagement;
import fr.mael.jiwigo.transverse.util.Messages;
import fr.mael.jiwigo.transverse.util.Tools;
import fr.mael.jiwigo.ui.field.HintTextField;
import fr.mael.jiwigo.ui.mainframe.DialogPrivacyLevel;
import fr.mael.jiwigo.ui.mainframe.MainFrame;
import fr.mael.jiwigo.ui.search.DialogChooseCategory;
import fr.mael.jiwigo.ui.search.File;

/**
    Copyright (c) 2010, Mael
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
     * Redistributions of source code must retain the above copyright
       notice, this list of conditions and the following disclaimer.
     * Redistributions in binary form must reproduce the above copyright
       notice, this list of conditions and the following disclaimer in the
       documentation and/or other materials provided with the distribution.
     * Neither the name of jiwigo nor the
       names of its contributors may be used to endorse or promote products
       derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
    ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL Mael BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    * @author mael
    * File tree
*/
public class FileTree extends JPanel implements TreeSelectionListener, MouseListener, ActionListener {
    private String path;

    private HintTextField fieldPath;

    private JButton refreshButton;

    private JPanel panelNorth;

    private DefaultMutableTreeNode root = new DefaultMutableTreeNode("");

    private JTree tree;

    private ArrayList<File> filesToSend;

    private File f;
    private File fls[];
    private ArrayList<File> filter;
    private File fls2[];
    private ArrayList<File> filter2 = new ArrayList<File>();
    /**
     * the menu to add a category
     */
    private JMenuItem menuSend;

    /**
     * Constructor
     * @param path : the path of the directory where to search for files
     */
    public FileTree(String path) {
	super(new BorderLayout());
	this.path = path;
	fieldPath = new HintTextField(Messages.getMessage("fileTree_clickToChoose"));
	if (path != null) {
	    fieldPath.setText(path);
	}
	fieldPath.addMouseListener(this);
	fieldPath.setPreferredSize(new Dimension(150, 30));
	refreshButton = new JButton();
	refreshButton.setIcon(new ImageIcon(Tools.getURL("fr/mael/jiwigo/img/refresh.png")));
	refreshButton.addActionListener(this);
	refreshButton.setFocusPainted(false);
	refreshButton.setBorderPainted(false);
	refreshButton.setContentAreaFilled(false);

	panelNorth = new JPanel(new FlowLayout());
	panelNorth.add(fieldPath);
	panelNorth.add(refreshButton);

	tree = new JTree(root);
	tree.getSelectionModel().setSelectionMode(TreeSelectionModel.DISCONTIGUOUS_TREE_SELECTION);
	tree.addTreeSelectionListener(this);
	tree.setCellRenderer(new MyTreeCellRenderer());
	tree.setRootVisible(false);
	tree.addMouseListener(this);
	JScrollPane treeView = new JScrollPane(tree);
	Dimension minimumSize = new Dimension(100, 50);
	treeView.setMinimumSize(minimumSize);
	this.add(treeView, BorderLayout.CENTER);
	this.add(panelNorth, BorderLayout.NORTH);
	reload();
    }

    /**
     * Method that reload the tree
     */
    public void reload() {
	if (path != null) {
	    root.removeAllChildren();
	    createNodes(root);
	    ((DefaultTreeModel) tree.getModel()).reload();
	}
    }

    /**
     * This method creates filters the files to display depending on their
     * extension (only image files)
     * @param root
     */
    private void createNodes(DefaultMutableTreeNode root) {
	DefaultMutableTreeNode fileNode = null;
	f = new File(path);
	try {
	    fls = null;
	    filter = null;
	    System.runFinalization();
	    fls = f.listFiles();
	    filter = new ArrayList<File>();
	    for (File file : fls) {
		//only png and jpg are displayed
		if ((file.isDirectory() && !(file.getName().charAt(0) == '.'))
			|| (file.isFile() && ((file.getName().toLowerCase().endsWith(".jpg")
				|| file.getName().toLowerCase().endsWith(".jpeg") || file.getName().toLowerCase()
				.endsWith(".png"))))) {
		    filter.add(file);
		}
	    }
	    //call to the method that creates the files hierarchy
	    for (File file : filter) {
		fileNode = new DefaultMutableTreeNode(file);
		root.add(fileNode);
		recursiveNodeCreation(fileNode, file);

	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}

    }

    /**
     * recursive method for the creation of the nodes
     * @param categoryNode
     * @param category
     */
    private void recursiveNodeCreation(DefaultMutableTreeNode categoryNode, File file) {
	if (file.isDirectory() && !(file.getName().charAt(0) == '.')) {
	    fls2 = null;
	    filter2 = null;
	    System.runFinalization();
	    fls2 = file.listFiles();
	    filter2 = new ArrayList<File>();
	    for (File fich : fls2) {
		if (!(fich.getName().charAt(0) == '.')
			&& fich.isDirectory()
			|| (fich.isFile() && ((fich.getName().toLowerCase().endsWith(".jpg")
				|| fich.getName().toLowerCase().endsWith(".jpeg") || fich.getName().toLowerCase()
				.endsWith(".png"))))) {
		    filter2.add(fich);
		}
	    }
	    for (File fich : filter2) {
		DefaultMutableTreeNode node = new DefaultMutableTreeNode(fich);
		categoryNode.add(node);
		recursiveNodeCreation(node, fich);
	    }
	}
    }

    @Override
    public void valueChanged(TreeSelectionEvent arg0) {

    }

    @Override
    public void mouseClicked(MouseEvent mouseEvent) {
	MainFrame.getInstance().showWaitCursor(true);
	if (mouseEvent.getSource().equals(fieldPath)) {
	    JFileChooser chooser = new JFileChooser();
	    if (path != null) {
		chooser.setCurrentDirectory(new java.io.File(path));
	    }
	    chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
	    chooser.setAcceptAllFileFilterUsed(false);
	    if (chooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
		path = chooser.getSelectedFile().getAbsolutePath();
		fieldPath.setText(path);
		reload();
	    }
	}

	if (mouseEvent.getButton() == 3) {
	    filesToSend = new ArrayList<File>();
	    TreePath treePath[] = tree.getSelectionPaths();
	    for (TreePath treeP : treePath) {
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) treeP.getLastPathComponent();
		File file = (File) node.getUserObject();
		filesToSend.add(file);
	    }
	    JPopupMenu popup = new JPopupMenu();
	    menuSend = new JMenuItem("Send");
	    menuSend.addActionListener(this);
	    popup.add(menuSend);
	    popup.show(tree, mouseEvent.getX(), mouseEvent.getY());
	}
	MainFrame.getInstance().showWaitCursor(false);
    }

    @Override
    public void mouseEntered(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mouseExited(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mousePressed(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    @Override
    public void mouseReleased(MouseEvent arg0) {
	// TODO Auto-generated method stub

    }

    public JTree getTree() {
	return tree;
    }

    @Override
    public void actionPerformed(ActionEvent paramActionEvent) {
	if (paramActionEvent.getSource().equals(menuSend)) {
	    //first, the dialog that allows to choose the privacy level of the photo(s) is called
	    if (!ImagesManagement.getInstance().isRememberPrivacyLevel()) {
		new DialogPrivacyLevel();
	    }
	    //then the dialog that allow to choose the category is called
	    new DialogChooseCategory(filesToSend);
	} else if (paramActionEvent.getSource().equals(refreshButton)) {
	    reload();
	}
    }
}
